@@ -1,4 +1,5 @@
 """
+Data management tools for storing and manipulating unstructured data
 """
 
 
@@ -59,7 +60,7 @@ class SteadyDataboxableProtocol(Protocol):
     #]
 
 
-def _extended_range_tuple_from_base_span(
+def _extended_span_tuple_from_base_span(
     input_span: Iterable[Period],
     min_shift: int,
     max_shift: int,
@@ -211,7 +212,7 @@ Databox, incorporating all its functionalities.
         descriptions: Iterable[str] | None = None,
         periods: Iterable[Period] | None = None,
         start: Period | None = None,
-        target_databox: Self | None = None,
+        target_db: Self | None = None,
         orientation: Literal["vertical", "horizontal", ] = "vertical",
     ) -> Self:
         """
@@ -230,7 +231,7 @@ of the numeric array.
         descriptions=None,
         periods=None,
         start=None,
-        target_databox=None,
+        target_db=None,
         orientation="vertical",
     )
 
@@ -254,7 +255,7 @@ of the numeric array.
 ???+ input "start"
     The start period for the time series data. Used if 'periods' is not provided.
 
-???+ input "target_databox"
+???+ input "target_db"
     An existing Databox to which the array data will be added. If `None`, a new 
     Databox is created.
 
@@ -281,7 +282,7 @@ of the numeric array.
             names,
             series_constructor,
             descriptions=descriptions,
-            target_databox=target_databox,
+            target_db=target_db,
         )
 
     @classmethod
@@ -292,11 +293,11 @@ of the numeric array.
         series_constructor: Callable,
         *,
         descriptions: Sequence[str] | None = None,
-        target_databox: Self | None = None,
+        target_db: Self | None = None,
     ) -> Self:
         """
         """
-        self = target_databox or klass()
+        self = target_db or klass()
         descriptions = (
             descriptions if descriptions is not None
             else _it.repeat("", )
@@ -352,7 +353,7 @@ No input arguments are required for this method.
         return tuple(self.keys())
 
     @_pages.reference(category="information", )
-    def get_missing_names(self, names: Iterable[str], /, ) -> tuple[str]:
+    def get_missing_names(self, names: Iterable[str], ) -> tuple[str]:
         """
 ················································································
 
@@ -403,7 +404,7 @@ yet to be added to the Databox.
         strict_names: bool = False,
     ) -> Self:
         """
-················································································
+................................................................................
 
 ==Create a copy of the Databox==
 
@@ -442,7 +443,7 @@ during the duplication process.
     A new Databox instance that is a deep copy of the current one, containing 
     either all items or only those specified.
 
-················································································
+................................................................................
         """
         new_databox = _co.deepcopy(self, )
         if source_names is None and target_names is None:
@@ -450,10 +451,22 @@ during the duplication process.
         source_names, target_names, *_ = self._resolve_source_target_names(
             source_names, target_names, strict_names,
         )
-        new_databox.rename(source_names, target_names, )
-        new_databox.keep(target_names, )
+        new_databox.rename(source_names, target_names, strict_names=strict_names, )
+        new_databox.keep(target_names, strict_names=strict_names, )
         return new_databox
 
+    def has(
+        self: Self,
+        /,
+        names: Iterable[str] | str,
+    ) -> bool:
+        """
+        """
+        return (
+            names in self if isinstance(names, str)
+            else not self.get_missing_names(names, )
+        )
+
     def shallow(
         self: Self,
         /,
@@ -471,7 +484,7 @@ during the duplication process.
             for s, t in zip(source_names, target_names, )
         )
 
-    def print_content(
+    def print_contents(
         self,
         source_names: SourceNames = None,
     ) -> None:
@@ -559,9 +572,8 @@ or a callable function taking a source name and returning the new target name.
 
 ### Returns ###
 
+Returns `None`; `self` is modified in place.
 
-???+ returns "None"
-    Alters the Databox in-place without returning a value.
 
 ················································································
         """
@@ -569,8 +581,6 @@ or a callable function taking a source name and returning the new target name.
             source_names, target_names, strict_names,
         )
         for s, t in zip(source_names, target_names, ):
-            if s == t:
-                continue
             self[t] = self.pop(s)
 
     @_pages.reference(category="manipulation", )
@@ -611,9 +621,8 @@ single name, a callable that returns `True` for names to be removed, or `None`.
 
 ### Returns ###
 
+Returns `None`; `self` is modified in place.
 
-???+ returns "None"
-    This method does not return any value but modifies the Databox in-place.
 
 ················································································
         """
@@ -1042,8 +1051,8 @@ Get the encompassing date span for all time series with a specified frequency.
     def steady(
         klass,
         steady_databoxable: SteadyDataboxableProtocol,
-        input_span: Iterable[Period],
-        /,
+        span: Iterable[Period],
+        *,
         deviation: bool = False,
         prepend_initial: bool = True,
         append_terminal: bool = True,
@@ -1051,15 +1060,15 @@ Get the encompassing date span for all time series with a specified frequency.
         """
         """
         self = klass()
-        start_date, end_date = _extended_range_tuple_from_base_span(
-            input_span,
+        start, end = _extended_span_tuple_from_base_span(
+            span,
             steady_databoxable.max_lag,
             steady_databoxable.max_lead,
             prepend_initial,
             append_terminal,
         )
         items = steady_databoxable.generate_steady_items(
-            start_date, end_date,
+            start, end,
             deviation=deviation,
         )
         for name, value in items:
@@ -1072,13 +1081,14 @@ Get the encompassing date span for all time series with a specified frequency.
         self,
         model,
         control: Self,
-        /,
     ) -> None:
         """
         """
         name_to_minus_control_func = model.map_name_to_minus_control_func()
         for name, func in name_to_minus_control_func.items():
+            description = self[name].get_description()
             self[name] = func(self[name], control[name], )
+            self[name].set_description(description, )
 
     def __or__(self, other) -> Self:
         new = _co.deepcopy(self)
@@ -1098,14 +1108,14 @@ Get the encompassing date span for all time series with a specified frequency.
         if source_names is None:
             source_names = context_names
         if isinstance(source_names, str):
-            source_names = tuple(source_names)
+            source_names = (source_names, )
         if callable(source_names):
             func = source_names
             source_names = tuple(n for n in context_names if func(n))
         if target_names is None:
             target_names = source_names
         if isinstance(target_names, str):
-            target_names = tuple(target_names)
+            target_names = (target_names, )
         if callable(target_names):
             func = target_names
             target_names = tuple(func(n) for n in source_names)
