@@ -6,13 +6,14 @@ Time periods and time spans
 #[
 from __future__ import annotations
 
-import re as _re
-import datetime as _dt
-import enum as _en
-import functools as _ft
 from typing import (Union, Self, Any, Protocol, TypeAlias, runtime_checkable, )
 from collections.abc import (Iterable, Callable, Iterator, )
 from numbers import (Real, )
+import re as _re
+import enum as _en
+import functools as _ft
+import datetime as _dt
+import calendar as _ca
 
 from . import pages as _pages
 from .conveniences import copies as _copies
@@ -28,6 +29,7 @@ __all__ = (
     "YEARLY", "HALFYEARLY", "QUARTERLY", "MONTHLY", "WEEKLY", "DAILY",
     "PERIOD_CLASS_FROM_FREQUENCY_RESOLUTION",
     "refrequent", "convert_to_new_freq",
+    "daily_serial_from_ymd",
 
     "daters_from_sdmx_strings", "daters_from_iso_strings", "daters_from_to",
     "Dater", "Ranger", "EmptyRanger",
@@ -393,7 +395,7 @@ def _period_constructor_with_ellipsis(
 )
 class Period(
     _SpannableMixin,
-    _copies.CopyMixin,
+    _copies.Mixin,
 ):
     """
 ......................................................................
@@ -657,6 +659,12 @@ literal.
         """
         return PERIOD_CLASS_FROM_FREQUENCY_RESOLUTION[freq].from_ymd(*args, )
 
+    @staticmethod
+    def from_year_period(freq: Frequency, *args, ) -> Self:
+        """
+        """
+        return PERIOD_CLASS_FROM_FREQUENCY_RESOLUTION[freq].from_year_period(*args, )
+
     period_from_ymd = from_ymd
     dater_from_ymd = from_ymd
 
@@ -1229,7 +1237,7 @@ class DailyPeriod(Period, ):
 
     @classmethod
     def from_ymd(klass: type, year: int, month: int=1, day: int=1) -> Self:
-        serial = _dt.date(year, month, day).toordinal()
+        serial = daily_serial_from_ymd(year, month, day, )
         return klass(serial)
 
     @classmethod
@@ -1367,10 +1375,13 @@ class RegularPeriodMixin:
     def to_ymd(
         self, 
         /,
-        position: Literal["start", "middle", "end", ] = "middle",
+        position: Literal["start", "middle", "end", ] = "start",
     ) -> tuple[int, int, int]:
         year, per = self.to_year_period()
-        return year, *self.month_day_resolution[position][per]
+        month, day = self._MONTH_DAY_RESOLUTION[position][per]
+        if day is None:
+            _, day = _ca.monthrange(year, month)
+        return year, month, day
 
     def __str__(self, /, ) -> str:
         return self.to_sdmx_string()
@@ -1396,7 +1407,7 @@ class RegularPeriodMixin:
     def to_daily(
         self,
         /,
-        position: Literal["start", "middle", "end", ] = "middle"
+        position: Literal["start", "middle", "end", ] = "start"
     ) -> DailyPeriod:
         try:
             return DailyPeriod.from_ymd(*self.to_ymd(position=position, ), )
@@ -1414,7 +1425,7 @@ class YearlyPeriod(RegularPeriodMixin, Period, ):
     frequency: Frequency = Frequency.YEARLY
     needs_resolve: bool = False
     origin = _serial_from_ypf(BASE_YEAR, 1, Frequency.YEARLY)
-    month_day_resolution = {
+    _MONTH_DAY_RESOLUTION = {
         "start": {1: (1, 1)},
         "middle": {1: (6, 30)},
         "end": {1: (12, 31)},
@@ -1442,7 +1453,7 @@ class HalfyearlyPeriod(RegularPeriodMixin, Period, ):
     frequency: Frequency = Frequency.HALFYEARLY
     needs_resolve: bool = False
     origin = _serial_from_ypf(BASE_YEAR, 1, Frequency.HALFYEARLY)
-    month_day_resolution = {
+    _MONTH_DAY_RESOLUTION = {
         "start": {1: (1, 1), 2: (7, 1)},
         "middle": {1: (3, 15), 2: (9, 15)},
         "end": {1: (6, 30), 2: (12, 31)}
@@ -1463,7 +1474,7 @@ class HalfyearlyPeriod(RegularPeriodMixin, Period, ):
     def to_ymd(
         self, 
         /,
-        position: Literal["start", "middle", "end", ] = "middle",
+        position: Literal["start", "middle", "end", ] = "start",
     ) -> tuple[int, int, int]:
         year, per = self.to_year_period()
         return (
@@ -1474,7 +1485,7 @@ class HalfyearlyPeriod(RegularPeriodMixin, Period, ):
     def get_month(
         self,
         /,
-        position: Literal["start", "middle", "end", ] = "middle",
+        position: Literal["start", "middle", "end", ] = "start",
     ) -> int:
         _, per = self.to_year_period()
         return month_resolution[position][per]
@@ -1493,7 +1504,7 @@ class QuarterlyPeriod(RegularPeriodMixin, Period, ):
     frequency: Frequency = Frequency.QUARTERLY
     needs_resolve: bool = False
     origin = _serial_from_ypf(BASE_YEAR, 1, Frequency.QUARTERLY)
-    month_day_resolution = {
+    _MONTH_DAY_RESOLUTION = {
         "start": {1: (1, 1), 2: (4, 1), 3: (7, 1), 4: (10, 1)},
         "middle": {1: (2, 15), 2: (5, 15), 3: (8, 15), 4: (11, 15)},
         "end": {1: (3, 31), 2: (6, 30), 3: (9, 30), 4: (12, 31)},
@@ -1523,10 +1534,10 @@ class MonthlyPeriod(RegularPeriodMixin, Period, ):
     frequency: Frequency = Frequency.MONTHLY
     needs_resolve: bool = False
     origin = _serial_from_ypf(BASE_YEAR, 1, Frequency.MONTHLY)
-    month_day_resolution = {
-        "start": {1: (1, 1), 2: (2, 1), 3: (2, 1), 4: (4, 1), 5: (5, 1), 6: (6, 1), 7: (7, 1), 8: (8, 1), 9: (9, 1), 10: (10, 1), 11: (11, 1), 12: (12, 1)},
-        "middle": {1: (1, 15), 2: (2, 15), 3: (2, 15), 4: (4, 15), 5: (5, 15), 6: (6, 15), 7: (7, 15), 8: (8, 15), 9: (9, 15), 10: (10, 15), 11: (11, 15), 12: (12, 15)},
-        "end": {1: (1, 31), 2: (2, 28), 3: (3, 31), 4: (4, 30), 5: (5, 31), 6: (6, 30), 7: (7, 31), 8: (8, 31), 9: (9, 30), 10: (10, 31), 11: (11, 30), 12: (12, 31)},
+    _MONTH_DAY_RESOLUTION = {
+        "start": { m: (m, 1) for m in range(1, 13) },
+        "middle": { m: (m, 15) for m in range(1, 13) },
+        "end": { m: ((m, _ca.monthrange(1970, m)[1]) if m != 2 else (m, None)) for m in range(1, 13) },
     }
 
     @classmethod
@@ -1658,6 +1669,9 @@ for n in ("yy", "hh", "qq", "mm", "dd", "ii", ):
     setattr(Period, n, locals()[n], )
 
 
+def daily_serial_from_ymd(year: int, month: int, day: int, ) -> int:
+    return _dt.date(year, month, day).toordinal()
+
 
 @_pages.reference(
     path=("data_management", "spans.md", ),
@@ -1670,7 +1684,7 @@ for n in ("yy", "hh", "qq", "mm", "dd", "ii", ):
     },
 )
 class Span(
-    _copies.CopyMixin,
+    _copies.Mixin,
 ):
     """
 ................................................................................
@@ -1731,6 +1745,12 @@ from a start period to an end period (possibly with a step size other than
         if not self.needs_resolve:
             _check_periods(from_per, until_per)
 
+    @classmethod
+    def encompassing(klass, *args, ) -> Self:
+        """
+        """
+        return get_encompassing_span(*args, )[0]
+
     @property
     @_pages.reference(category="property", )
     def start(self):
@@ -1846,7 +1866,8 @@ depending on the direction and magnitude of the shift.
 
 ### Returns ###
 
-This method modifies the object in place and does not return a value.
+???+ returns "None"
+    This method modifies `self` in-place and does not return a value.
 
 ................................................................................
         """
@@ -2110,7 +2131,9 @@ position in the timeline.
 
 ### Returns ###
 
-This method modifies the object in place and does not return a value.
+???+ returns "None"
+    This method modifies `self` in-place and does not return a value.
+
 
 ................................................................................
         """
@@ -2267,16 +2290,18 @@ def periods_from_iso_strings(
     return tuple(period_class.from_iso_string(i, ) for i in iso_strings)
 
 
-def get_encompassing_span(*args: ResolutionContextProtocol, ) -> Span:
+def get_encompassing_span(
+    *args: tuple[ResolutionContextProtocol | None, ...],
+) -> tuple[Span, Period, Period, ]:
     """
     """
-    start_dates = tuple(_get_period(x, "start_date", min, ) for x in args)
-    end_periods = tuple( _get_period(x, "end_date", max, ) for x in args)
-    start_dates = tuple(d for d in start_dates if d is not None)
-    end_periods = tuple(d for d in end_periods if d is not None)
+    start_dates = tuple(_get_period(i, "start_date", min, ) for i in args if i is not None)
+    end_periods = tuple( _get_period(i, "end_date", max, ) for i in args if i is not None)
+    start_dates = tuple(i for i in start_dates if i is not None)
+    end_periods = tuple(i for i in end_periods if i is not None)
     start = min(start_dates) if start_dates else None
     end = max(end_periods) if end_periods else None
-    return (Span(start, end), start, end, )
+    return Span(start, end), start, end,
 
 
 def _get_period(something, attr_name, select_func, ) -> Period | None:
