@@ -47,21 +47,48 @@ Table of contents
 from __future__ import annotations
 
 from typing import (Self, Any, Protocol, )
+from types import (SimpleNamespace, )
 from collections.abc import (Iterable, )
 import itertools as _it
 import functools as _ft
 import dataclasses as _dc
 import numpy as _np
 
+from ..incidences.main import (Token, )
 from ..incidences import main as _incidence
 from .. import equations as _equations
 from .. import quantities as _quantities
-from ..aldi import differentiators as _differentiators
+from ..aldi.differentiators import (AtomFactoryProtocol, Context, )
 from ..aldi import maps as _maps
 from .. import sources as _sources
 #]
 
 
+# Implement AtomFactoryProtocol
+
+def _create_diff_for_token(
+    token: Token,
+    wrt_tokens: tuple[Token, ...],
+    /,
+) -> _np.ndarray:
+    """
+    """
+    try:
+        index = wrt_tokens.index(token, )
+    except ValueError:
+        return 0
+    diff = _np.zeros((len(wrt_tokens), 1, ), dtype=float, )
+    diff[index] = 1
+    return diff
+
+
+_ATOM_FACTORY = SimpleNamespace(
+    create_data_index_for_token=lambda token: (token.qid, token.shift, ),
+    create_diff_for_token=_create_diff_for_token,
+    get_diff_shape=lambda wrt_tokens: (len(wrt_tokens), 1, ),
+)
+
+
 class Descriptor:
     """
     Descriptor of first-order systems and solutions, and aldi Atom factory provider
@@ -101,9 +128,8 @@ class Descriptor:
         )
         #
         # Create the evaluation context for the algorithmic differentiator
-        atom_factory = self
-        self.aldi_context = _differentiators.Context(
-            atom_factory,
+        self.aldi_context = Context(
+            _ATOM_FACTORY,
             system_equations,
             eid_to_wrts=self.system_vectors.eid_to_wrt_tokens,
             qid_to_logly=_quantities.create_qid_to_logly(quantities),
@@ -116,34 +142,6 @@ class Descriptor:
     def get_num_forwards(self: Self) -> int:
         return self.system_vectors.get_num_forwards()
 
-    # ===== Implement AtomFactoryProtocol =====
-
-    def create_diff_for_token(
-        self,
-        token: Token,
-        wrt_tokens: tuple[Token, ...],
-        /,
-    ) -> _np.ndarray:
-        """
-        """
-        if token is None:
-            return _np.zeros((len(wrt_tokens), 1, ), )
-        try:
-            index = wrt_tokens.index(token, )
-            diff = _np.zeros((len(wrt_tokens), 1, ), )
-            diff[index] = 1
-            return diff
-        except:
-            return 0
-
-    def create_data_index_for_token(
-        self,
-        token: Token,
-    ) -> tuple[int, slice]:
-        """
-        """
-        return (token.qid, token.shift, )
-
     #]
 
 
@@ -184,16 +182,16 @@ class SystemVectors:
     #[
     transition_eids: Iterable[int] | None = None
     measurement_eids: Iterable[int] | None = None
-    eid_to_wrt_tokens: dict[int, Iterable[_incidence.Token]] | None = None
+    eid_to_wrt_tokens: dict[int, Iterable[Token]] | None = None
     #
-    transition_variables: Iterable[_incidence.Token] | None = None
+    transition_variables: Iterable[Token] | None = None
     transition_variables_are_logly: list[bool] | None = None
-    are_initial_conditions: list[bool] | None = None,
-    unanticipated_shocks: tuple[_incidence.Token, ...] | None = None
-    anticipated_shocks: tuple[_incidence.Token, ...] | None = None
-    measurement_variables: Iterable[_incidence.Token] | None = None
+    true_initials: list[bool] | None = None,
+    unanticipated_shocks: tuple[Token, ...] | None = None
+    anticipated_shocks: tuple[Token, ...] | None = None
+    measurement_variables: Iterable[Token] | None = None
     measurement_variables_are_logly: list[bool] | None = None
-    measurement_shocks: tuple[_incidence.Token, ...] | None = None
+    measurement_shocks: tuple[Token, ...] | None = None
     #
     shape_A_excl_dynid: tuple[int, int] | None = None
     shape_B_excl_dynid: tuple[int, int] | None = None
@@ -221,11 +219,16 @@ class SystemVectors:
         self.measurement_eids \
             = sorted([eqn.id for eqn in equations if eqn.kind in _equations.EquationKind.MEASUREMENT_EQUATION])
         qid_to_kind = _quantities.create_qid_to_kind(quantities, )
-        all_tokens = set(_equations.generate_all_tokens_from_equations(equations))
+        #
+        # Collect all tokens from equations but also add zero-shifted tokens
+        # created from all variables, shocks, and parameters to catch any names
+        # not used in equations (shocks, parameters)
+        all_tokens = _collect_all_tokens(equations, quantities, )
+        #
         all_wrt_tokens = set(_incidence.generate_tokens_of_kinds(all_tokens, qid_to_kind, _SYSTEM_QUANTITY))
         self.eid_to_wrt_tokens = _equations.create_eid_to_wrt_tokens(equations, all_wrt_tokens)
         #
-        actual_tokens_transition_variables \
+        actual_transition_variable_tokens \
             = set(_incidence.generate_tokens_of_kinds(all_tokens, qid_to_kind, _quantities.QuantityKind.TRANSITION_VARIABLE))
         #
         # Make adjustment for transition variables in measurement
@@ -233,17 +236,14 @@ class SystemVectors:
         # dated (LHS) vector of transition variables; this is done by
         # pretending x(t-k-1) is needed
         #
-        adjusted_tokens_transition_variables \
-            = _adjust_for_measurement_equations(actual_tokens_transition_variables, equations, qid_to_kind)
+        adjusted_transition_variable_tokens \
+            = _adjust_for_measurement_equations(actual_transition_variable_tokens, equations, qid_to_kind)
         #
         self.transition_variables \
-            = _incidence.sort_tokens(_create_system_transition_vector(adjusted_tokens_transition_variables))
+            = _incidence.sort_tokens(_create_system_transition_vector(adjusted_transition_variable_tokens))
         self.transition_variables_are_logly \
             = [ qid_to_logly[tok.qid] for tok in self.transition_variables ]
-        self.are_initial_conditions = [
-            _incidence.Token(t.qid, t.shift-1) in actual_tokens_transition_variables and t.shift <= 0
-            for t in self.transition_variables
-        ]
+        self.populate_true_initials(actual_transition_variable_tokens, )
         #
         # Unanticipated shocks
         #
@@ -292,6 +292,23 @@ class SystemVectors:
 
     def get_num_forwards(self) -> int:
         return _get_num_forwards(self.transition_variables)
+
+    def populate_true_initials(
+        self,
+        actual_transition_variable_tokens: set[Token],
+    ) -> None:
+        """
+        """
+        # Get the maximum lag for each transition variable
+        # actual_min_shifts[qid] = min(shifts) across all tokens with qid
+        actual_min_shifts = _incidence.get_some_shift_by_quantities(actual_transition_variable_tokens, min, )
+        def is_true_initial(token, ):
+            shift = token.shift - 1
+            min_shift = actual_min_shifts[token.qid]
+            return min_shift <= shift < 0
+        self.true_initials = tuple(
+            is_true_initial(t) for t in self.transition_variables
+        )
     #]
 
 
@@ -301,19 +318,19 @@ class SolutionVectors:
     Vectors of quantities in first-order solution matrices
     """
     #[
-    transition_variables: tuple[_incidence.Token, ...] | None = None
-    are_initial_conditions: list[bool, ...] | None = None
-    unanticipated_shocks: tuple[_incidence.Token, ...] | None = None
-    anticipated_shocks: tuple[_incidence.Token, ...] | None = None
-    measurement_variables: tuple[_incidence.Token, ...] | None = None
-    measurement_shocks: tuple[_incidence.Token, ...] | None = None
+    transition_variables: tuple[Token, ...] | None = None
+    true_initials: list[bool, ...] | None = None
+    unanticipated_shocks: tuple[Token, ...] | None = None
+    anticipated_shocks: tuple[Token, ...] | None = None
+    measurement_variables: tuple[Token, ...] | None = None
+    measurement_shocks: tuple[Token, ...] | None = None
 
     def __init__(self, system_vectors: SystemVectors, /, ) -> None:
         """
         Construct solution vectors and initial conditions indicator
         """
-        self.transition_variables, self.are_initial_conditions = \
-            _solution_vector_from_system_vector(system_vectors.transition_variables, system_vectors.are_initial_conditions)
+        self.transition_variables, self.true_initials = \
+            _solution_vector_from_system_vector(system_vectors.transition_variables, system_vectors.true_initials)
         self.unanticipated_shocks = tuple(system_vectors.unanticipated_shocks)
         self.anticipated_shocks = tuple(system_vectors.anticipated_shocks)
         self.measurement_variables = tuple(system_vectors.measurement_variables)
@@ -322,11 +339,11 @@ class SolutionVectors:
     def get_initials(
         self,
         /,
-    ) -> list[_incidence.Token]:
+    ) -> list[Token]:
         """
         Get tokens representing required initial conditions
         """
-        return list(_it.compress(self.transition_variables, self.are_initial_conditions))
+        return list(_it.compress(self.transition_variables, self.true_initials))
 
     def get_curr_transition_indexes(
         self,
@@ -373,29 +390,29 @@ class HumanSolutionVectors:
 
 
 def _create_system_transition_vector(
-    tokens_transition_variables: Iterable[_incidence.Token],
+    transition_variable_tokens: Iterable[Token],
     /,
-) -> Iterable[_incidence.Token]:
+) -> Iterable[Token]:
     """
     From tokens of transition variables, create vector of transition variables
     along columns of matrix A in unsolved system
     """
     #[
-    tokens_transition_variables = set(tokens_transition_variables)
-    min_shifts = _incidence.get_some_shift_by_quantities(tokens_transition_variables, lambda x: min(min(x), -1))
-    max_shifts = _incidence.get_some_shift_by_quantities(tokens_transition_variables, max)
+    transition_variable_tokens = set(transition_variable_tokens)
+    min_shifts = _incidence.get_some_shift_by_quantities(transition_variable_tokens, lambda x: min(min(x), -1))
+    max_shifts = _incidence.get_some_shift_by_quantities(transition_variable_tokens, max)
     #
-    vector_for_id = lambda qid: [_incidence.Token(qid, sh) for sh in range(min_shifts[qid]+1, max_shifts[qid]+1)]
-    unique_ids = set(t.qid for t in tokens_transition_variables)
-    return _it.chain.from_iterable(vector_for_id(i) for i in unique_ids)
+    vector_for_qid = lambda qid: [Token(qid, sh) for sh in range(min_shifts[qid]+1, max_shifts[qid]+1)]
+    unique_qids = set(t.qid for t in transition_variable_tokens)
+    return _it.chain.from_iterable(vector_for_qid(i) for i in unique_qids)
     #]
 
 
 def _solution_vector_from_system_vector(
-    system_transition_vector: Iterable[_incidence.Token],
-    are_initial_conditions: Iterable[bool],
+    system_transition_vector: Iterable[Token],
+    true_initials: Iterable[bool],
     /,
-) -> Iterable[_incidence.Token]:
+) -> Iterable[Token]:
     """
     From sorted system vector, get vector of transition variables in solved
     system and the indicator of required initial conditions
@@ -403,18 +420,18 @@ def _solution_vector_from_system_vector(
     num_forwards = _get_num_forwards(system_transition_vector)
     return (
         tuple(system_transition_vector[num_forwards:]),
-        tuple(are_initial_conditions[num_forwards:]),
+        tuple(true_initials[num_forwards:]),
     )
 
 
-def _get_num_forwards(system_transition_vector: Iterable[_incidence.Token]):
+def _get_num_forwards(system_transition_vector: Iterable[Token]):
     """
     Number of forward-looking tokens in a vector of tokens
     """
     return sum(1 for t in system_transition_vector if t.shift>0)
 
 
-def _get_num_backwards(system_transition_vector: Iterable[_incidence.Token]):
+def _get_num_backwards(system_transition_vector: Iterable[Token]):
     """
     Number of backward-looking tokens in a vector of tokens
     """
@@ -457,7 +474,7 @@ class SystemMap:
         #
         # Transition equations
         #
-        self.A = _maps.ArrayMap(
+        self.A = _maps.ArrayMap.static(
             system_vectors.transition_eids,
             system_vectors.eid_to_wrt_tokens,
             system_vectors.transition_variables,
@@ -476,7 +493,7 @@ class SystemMap:
             for t in lagged_transition_variables
         ]
         #
-        self.B = _maps.ArrayMap(
+        self.B = _maps.ArrayMap.static(
             system_vectors.transition_eids,
             system_vectors.eid_to_wrt_tokens,
             lagged_transition_variables,
@@ -488,9 +505,9 @@ class SystemMap:
         self.A.remove_nones()
         self.B.remove_nones()
         #
-        self.C = _maps.VectorMap(system_vectors.transition_eids, )
+        self.C = _maps.VectorMap.static(system_vectors.transition_eids, )
         #
-        self.D = _maps.ArrayMap(
+        self.D = _maps.ArrayMap.static(
             system_vectors.transition_eids,
             system_vectors.eid_to_wrt_tokens,
             system_vectors.unanticipated_shocks,
@@ -499,7 +516,7 @@ class SystemMap:
             lhs_column_offset=0,
         )
         #
-        self.E = _maps.ArrayMap(
+        self.E = _maps.ArrayMap.static(
             system_vectors.transition_eids,
             system_vectors.eid_to_wrt_tokens,
             system_vectors.anticipated_shocks,
@@ -516,7 +533,7 @@ class SystemMap:
         #
         # Measurement equations
         #
-        self.F = _maps.ArrayMap(
+        self.F = _maps.ArrayMap.static(
             system_vectors.measurement_eids,
             system_vectors.eid_to_wrt_tokens,
             system_vectors.measurement_variables,
@@ -525,7 +542,7 @@ class SystemMap:
             lhs_column_offset=0,
         )
         #
-        self.G = _maps.ArrayMap(
+        self.G = _maps.ArrayMap.static(
             system_vectors.measurement_eids,
             system_vectors.eid_to_wrt_tokens,
             system_vectors.transition_variables,
@@ -534,9 +551,9 @@ class SystemMap:
             lhs_column_offset=0,
         )
         #
-        self.H = _maps.VectorMap(system_vectors.measurement_eids, )
+        self.H = _maps.VectorMap.static(system_vectors.measurement_eids, )
         #
-        self.J = _maps.ArrayMap(
+        self.J = _maps.ArrayMap.static(
             system_vectors.measurement_eids,
             system_vectors.eid_to_wrt_tokens,
             system_vectors.measurement_shocks,
@@ -547,7 +564,7 @@ class SystemMap:
     #]
 
 
-def _create_dynid_matrices(system_transition_vector: Iterable[_incidence.Token]):
+def _create_dynid_matrices(system_transition_vector: Iterable[Token]):
     """
     Create dynamic identity matrix for unsolved system
     """
@@ -577,7 +594,7 @@ def _create_dynid_matrices(system_transition_vector: Iterable[_incidence.Token])
 
 
 def _adjust_for_measurement_equations(
-    tokens_transition_variables: Iterable[_quantities.Quantity],
+    transition_variable_tokens: Iterable[_quantities.Quantity],
     equations: Iterable[_equations.Equation],
     qid_to_kind: dict[int, _quantities.QuantityKind],
     /,
@@ -587,10 +604,10 @@ def _adjust_for_measurement_equations(
     #[
     tokens_in_measurement_equations = _it.chain.from_iterable(e.incidence for e in equations if e.kind is _equations.EquationKind.MEASUREMENT_EQUATION)
     pretend_needed = [
-        _incidence.Token(t.qid, t.shift-1) for t in tokens_in_measurement_equations
+        Token(t.qid, t.shift-1) for t in tokens_in_measurement_equations
         if qid_to_kind[t.qid] in _quantities.QuantityKind.TRANSITION_VARIABLE
     ]
-    return set(tokens_transition_variables).union(pretend_needed)
+    return set(transition_variable_tokens).union(pretend_needed)
     #]
 
 
@@ -697,3 +714,24 @@ class Squid:
     #]
 
 
+def _collect_all_tokens(
+    equations: Iterable[_equations.Equation],
+    quantities: Iterable[_quantities.Quantity],
+    /,
+) -> set[Token]:
+    """
+    Combine all tokens from equations and zero-shifted tokens created from
+    variables, shocks and parameters.
+    """
+    #[
+    # Tokens from equations
+    tokens_from_equations \
+        = set(_equations.generate_all_tokens_from_equations(equations, ))
+    #
+    # Zero-shift tokens from all variables, shocks and parameters
+    kind = _quantities.ANY_VARIABLE | _quantities.ANY_SHOCK | _quantities.PARAMETER
+    tokens_from_quantities \
+        = set(_incidence.generate_zero_shift_tokens_from_quantities(quantities, kind=kind))
+    return tokens_from_equations | tokens_from_quantities
+    #]
+
