@@ -11,7 +11,7 @@ Table of contents
 * `simulate_measurement`
 * `_generate_period_system`
 * `_store_smooth`
-* `_get_init_xi`
+* `get_init_xi`
 
 """
 
@@ -30,11 +30,17 @@ from ..dataslates.main import (Dataslate, )
 from ..plans.simulation_plans import (SimulationPlan, )
 from .. import frames as _frames
 from ..incidences import main as _incidences
+from ..incidences.main import (Token, )
 from .solutions import (Solution, )
 from .descriptors import (SolutionVectors, Squid, )
 from .kalmans import (Cache, )
 from . import kalmans as _kalmans
 from . import shock_simulators as _shock_simulators
+
+from typing import (TYPE_CHECKING, )
+if TYPE_CHECKING:
+    from typing import (Any, )
+    from collections.abc import (Iterable, )
 #]
 
 
@@ -46,22 +52,20 @@ _simulate_anticipated_shocks \
     = _shock_simulators.simulate_square_anticipated_shocks
 
 
-class _FordSimulatableProtocol(Protocol, ):
+class FordSimulatableProtocol(Protocol, ):
     """
     """
     #[
 
     num_variants: int
-
     solution_vectors: SolutionVectors
-
     def get_singleton_solution(self, deviation: bool, ) -> Solution: ...
 
     #]
 
 
 def simulate(
-    model_v: _FordSimulatableProtocol,
+    model_v: FordSimulatableProtocol,
     dataslate_v: Dataslate,
     plan: SimulationPlan | None,
     vid: int,
@@ -71,6 +75,7 @@ def simulate(
     deviation: bool = False,
     check_singularity: bool = False,
     force_split_frames: bool = False,
+    return_info: bool = False,
 ) -> dict[str, Any]:
     """
     """
@@ -80,19 +85,11 @@ def simulate(
     data_array = dataslate_v.get_data_variant(0, )
     squid = Squid(model_v, )
     #
-    #
-    is_plan_empty = plan is None or plan.is_empty
-    if is_plan_empty:
-        plan_registers = None
-        curr_xi_qids = None
-    else:
-        plan_registers = _get_plan_registers_as_bool_arrays(plan, periods, )
-        curr_xi_qids, *_ = model_v.solution_vectors.get_curr_transition_indexes()
-    #
     # In first-order simulation, splitting into frames is necessary only if
     # there are some endogenized anticipated shocks and, at the same time, some
     # nonzero unanticipated shocks (not counting the first period in either case)
     #
+    is_plan_empty = plan is None or plan.is_empty
     needs_splitting = _check_needs_splitting(
         plan=plan,
         dataslate=dataslate_v,
@@ -110,6 +107,7 @@ def simulate(
     if is_plan_empty:
         simulate_frame = simulate_flat
     else:
+        plan_registers = _get_plan_registers_as_bool_arrays(plan, periods, )
         simulate_frame = _ft.partial(
             simulate_conditional,
             check_singularity=check_singularity,
@@ -132,29 +130,49 @@ def simulate(
     logger.debug(f"Splitting simulation into {len(frames)} frame(s)")
     #
     # Resolve columns within each frame based on the start date of the dataslate
-    #
     for frame in frames:
         frame.resolve_columns(periods[0], )
     #
     #
+    #===========================================================================
     xi_array = _np.full((squid.num_xi, num_periods, ), _np.nan, )
+    frame_db = ()
     for frame in frames:
+        #
+        if needs_splitting:
+            frame_ds = dataslate_v.copy(invariant=False, )
+            frame.reset_unanticipated(frame_ds, model_v, )
+        else:
+            frame_ds = dataslate_v
+        #
         xi_array_in_frame = simulate_frame(
-            model_v, dataslate_v, frame, vid,
+            model_v, frame_ds, frame, vid,
             logger=logger,
             deviation=deviation,
         )
         xi_array[:, frame.slice] = xi_array_in_frame[:, frame.slice]
-
-    simulate_measurement(
-        model_v, dataslate_v, frame, xi_array, vid,
-        logger=logger,
-        deviation=deviation,
-    )
+        #
+        simulate_measurement(
+            model_v, dataslate_v, frame, xi_array, vid,
+            logger=logger,
+            deviation=deviation,
+        )
+        #
+        # Copy frame data back to the main dataslate if there are more than one
+        # frame; otherwise the main dataslate is the same as the frame dataslate
+        if needs_splitting:
+            frame.copy_frame_data(dataslate_v, frame_ds, )
+        #
+        # Create a frame databox only if output info is requested
+        if return_info:
+            frame_db = frame_db + (frame_ds.to_databox(), )
+    #===========================================================================
+    #
     #
     info = {
         "method": "first_order",
         "frames": frames,
+        "frame_db": frame_db,
     }
     #
     return info
@@ -162,7 +180,7 @@ def simulate(
 
 
 def simulate_flat(
-    model_v: _FordSimulatableProtocol,
+    model_v: FordSimulatableProtocol,
     dataslate_v: Dataslate,
     frame: Frame,
     vid: int,
@@ -177,24 +195,15 @@ def simulate_flat(
     solution = model_v.get_singleton_solution(deviation=deviation, )
     vec = model_v.solution_vectors
     num_periods = dataslate_v.num_periods
-    #
-    dataslate_v.logarithmize()
-    data_array = dataslate_v.get_data_variant(0, )
+    simulation_columns = tuple(range(*frame.simulation_slice.indices(num_periods, ), ), )
+    curr_xi_qids, curr_xi_indexes = vec.get_curr_transition_indexes()
     #
     T = solution.T
     P = solution.P
-    R = solution.R
     K = solution.K
     #
-    Z = solution.Z
-    H = solution.H
-    D = solution.D
-    #
-    xi = _get_init_xi(data_array, frame.first, vec, )
-    curr_xi_qids, curr_xi_indexes = vec.get_curr_transition_indexes()
-    #
-    # Prepare array of all xi values (needed for measurement simulation)
-    #
+    dataslate_v.logarithmize()
+    data_array = dataslate_v.get_data_variant(0, )
     num_xi = len(vec.transition_variables)
     xi_array = _np.full((num_xi, num_periods, ), _np.nan, )
     u_array = _extract_shock_values(data_array, vec.unanticipated_shocks, )
@@ -204,7 +213,9 @@ def simulate_flat(
     # Store results in data_array
     # Capture xi_array for later use
     #
-    for t in range(*frame.simulation_slice.indices(num_periods, ), ):
+    xi = get_init_xi(data_array, vec.transition_variables, simulation_columns[0], )
+    zero_false_init_xi(xi, vec.true_initials, )
+    for t in simulation_columns:
         xi = T @ xi + P @ u_array[:, t] + K
         if all_v_impact[t] is not None:
             xi += all_v_impact[t]
@@ -212,12 +223,13 @@ def simulate_flat(
         data_array[curr_xi_qids, t] = xi[curr_xi_indexes, ...]
     #
     dataslate_v.delogarithmize()
+    #
     return xi_array
     #]
 
 
 def simulate_conditional(
-    model_v: _FordSimulatableProtocol,
+    model_v: FordSimulatableProtocol,
     dataslate_v: Dataslate,
     frame: Frame,
     vid: int,
@@ -250,7 +262,8 @@ def simulate_conditional(
     #
     # Initialize Kalman filter:
     # [initial median, initial MSE, number of initials to estimates, ]
-    init_med = _get_init_xi(data_array, frame.first, vec, )
+    init_med = get_init_xi(data_array, vec.transition_variables, frame.first, )
+    zero_false_init_xi(init_med, vec.true_initials, )
     init_mse = _np.zeros((squid.num_xi, squid.num_xi), )
     unknown_init_impact = None
     initials = (init_med, init_mse, unknown_init_impact, )
@@ -345,7 +358,7 @@ def simulate_conditional(
 
 
 def simulate_measurement(
-    model_v: _FordSimulatableProtocol,
+    model_v: FordSimulatableProtocol,
     dataslate_v: Dataslate,
     frame: Frame,
     xi_array: _np.ndarray,
@@ -382,7 +395,7 @@ def simulate_measurement(
 
 def _generate_period_system(
     t: int,
-
+    #
     initials: list[_np.ndarray, _np.ndarray, int, ],
     solution: Solution,
     Z_xi: _np.ndarray | None,
@@ -525,7 +538,7 @@ def _store_smooth(
     xi_array: _np.ndarray,
     incidence_v: _np.ndarray | None,
     squid: Squid,
-
+    #
     t: int,
     xi: _np.ndarray | None = None,
     u: _np.ndarray | None = None,
@@ -557,30 +570,30 @@ def _store_smooth(
     #]
 
 
-def _get_init_xi(
+def get_init_xi(
     maybelog_working_data: _np.ndarray,
+    transition_solution_vector: Iterable[Token, ...],
     first_column: int,
-    solution_vectors: SolutionVectors,
-    /,
 ) -> _np.ndarray:
     """
     """
     #[
-    init_xi_rows, init_xi_columns \
-        = _incidences.rows_and_columns_from_tokens(
-            solution_vectors.transition_variables,
-            first_column - 1,
-        )
-    init_xi = maybelog_working_data[init_xi_rows, init_xi_columns]
-    unnecessary_initials = ~_np.array(solution_vectors.are_initial_conditions, )
-    init_xi[unnecessary_initials] = 0
-    return init_xi
+    init_xi_rows, init_xi_columns = _incidences.rows_and_columns_from_tokens(
+        transition_solution_vector,
+        first_column - 1,
+    )
+    return maybelog_working_data[init_xi_rows, init_xi_columns]
     #]
 
 
-def _get_xi_indexes(solution_vectors, ):
-    """
-    """
+def zero_false_init_xi(
+    init_xi: _np.ndarray,
+    true_initials: Iterable[bool, ...],
+) -> None:
+    #[
+    false_initials = [ (not i) for i in true_initials ]
+    init_xi[false_initials, ...] = 0
+    #]
 
 
 def _get_plan_registers_as_bool_arrays(
@@ -624,11 +637,15 @@ def _insert_exogenized_unanticipated(
 ) -> None:
     """
     """
+    #[
     plan_registers, input_data_array, squid, frame_slice, logger = packed
+    #
     ___ = plan_registers["exogenized_unanticipated"][:, frame_slice]
     curr_xi_exogenized[:, frame_slice][___] \
         = input_data_array[squid.curr_xi_qids, frame_slice][___]
+    #
     logger.debug(f"Exogenized unanticipated: {___.sum()}")
+    #]
 
 
 def _insert_exogenized_anticipated(
@@ -637,11 +654,15 @@ def _insert_exogenized_anticipated(
 ) -> None:
     """
     """
+    #[
     plan_registers, input_data_array, squid, frame_slice, logger = packed
+    #
     ___ = plan_registers["exogenized_anticipated"][:, frame_slice]
     curr_xi_exogenized[:, frame_slice][___] \
         = input_data_array[squid.curr_xi_qids, frame_slice][___]
+    #
     logger.debug(f"Exogenized anticipated: {___.sum()}")
+    #]
 
 
 def _insert_endogenized_unanticipated(
@@ -650,11 +671,16 @@ def _insert_endogenized_unanticipated(
 ) -> None:
     """
     """
+    #[
     plan_registers, input_data_array, squid, frame_slice, logger = packed
+    #
     ___ = plan_registers["endogenized_unanticipated"][:, frame_slice]
     std_u_endogenized[:, frame_slice][___] \
         = input_data_array[squid.std_u_qids, frame_slice][___]
+    #
     logger.debug(f"Endogenized unanticipated: {___.sum()}")
+    #]
+
 
 
 def _block_diag(A, B):
@@ -705,3 +731,4 @@ def _check_needs_splitting(
     return True
     #]
 
+
