@@ -16,6 +16,7 @@ import itertools as _it
 import functools as _ft
 import operator as _op
 import copy as _cp
+import textwrap as _tw
 
 from ..conveniences import descriptions as _descriptions
 from ..conveniences import copies as _copies
@@ -31,6 +32,7 @@ from . import _filling
 from . import _moving
 from . import _conversions
 from . import _hp
+from . import _ar
 from . import _x13
 
 from . import _indexing
@@ -47,6 +49,9 @@ from ._filling import *
 from ._hp import __all__ as _hp__all__
 from ._hp import *
 
+from ._ar import __all__ as _ar__all__
+from ._ar import *
+
 from ._x13 import __all__ as _x13__all__
 from ._x13 import *
 
@@ -73,24 +78,29 @@ _ELEMENTWISE_FUNCTIONS = {
     "normal_pdf": _sp.stats.norm.pdf,
 }
 
+FUNCTION_ADAPTATIONS_TIMEWISE_NUMPY = (
+    "sum",
+    "mean",
+)
+
 
 # FIXME
 FUNCTION_ADAPTATIONS_ELEMENTWISE = tuple(_ELEMENTWISE_FUNCTIONS.keys())
+
 FUNCTION_ADAPTATIONS_NUMPY_APPLY = () # ("maximum", "minimum", "mean", "median", "abs", )
 FUNCTION_ADAPTATIONS_BUILTINS = ("max", "min", )
 FUNCTION_ADAPTATIONS = tuple(set(
     FUNCTION_ADAPTATIONS_ELEMENTWISE
-    # + FUNCTION_ADAPTATIONS_NUMPY
-    # + FUNCTION_ADAPTATIONS_BUILTINS
 ))
 
 
 __all__ = (
-    ("Series", "shift", )
+    ("Series", "shift", "series", )
     + _conversions__all__
     + _diffs_cums__all__
     + _fillings__all__
     + _hp__all__
+    + _ar__all__
     + _moving__all__
     + _x13__all__
     + FUNCTION_ADAPTATIONS
@@ -118,6 +128,8 @@ def _get_date_positions(dates, base, num_periods, /, ):
     categories={
         "constructor": "Constructing new time series",
         "conversion": "Converting time series frequency",
+        "manipulation": "Manipulating time series values",
+        "homogenizing": "Homogenizing and extrapolating time series",
         "filtering": "Filtering time series",
         "moving": "Applying moving window functions",
         "temporal_change": "Calculating temporal change",
@@ -130,13 +142,14 @@ class Series(
     _temporal.Inlay,
     _filling.Inlay,
     _hp.Inlay,
+    _ar.Inlay,
     _moving.Inlay,
     _x13.Inlay,
     _plotly.Inlay,
-
+    _views.Inlay,
+    #
     _descriptions.DescriptionMixin,
-    _views.ViewMixin,
-    _copies.CopyMixin,
+    _copies.Mixin,
 ):
     r"""
 ················································································
@@ -158,7 +171,7 @@ variants of the data, stored as mutliple columns.
         "data",
         "data_type",
         "metadata",
-        "_description", 
+        "_description",
     )
 
     _numeric_format: str = "15g"
@@ -186,6 +199,7 @@ variants of the data, stored as mutliple columns.
         frequency: Frequency | None = None,
         values: Any | None = None,
         func: Callable | None = None,
+        populate: bool = True,
     ) -> None:
         """
 ················································································
@@ -229,31 +243,48 @@ self = Series(
 ### Returns ###
 
 
-???+ returns "self"
+???+ returns "None"
+    This method modifies `self` in-place and does not return a value.
 
-    The newly created `Series` object.
 
 ················································································
         """
         self.start = None
         self.data_type = data_type
-        self.data = _np.full((0, num_variants), self._missing, dtype=self.data_type)
+        self.data = _np.full((0, num_variants), _np.nan, dtype=self.data_type, )
         self.metadata = {}
         self._description = description
         #
         start = start_date if start is None else start
         periods = dates if periods is None else periods
         #
-        test = (x is not None for x in (start, periods, values, func))
-        populator = _SERIES_POPULATOR.get(tuple(test), _invalid_constructor)
-        populator(
-            self,
-            start=start,
-            periods=periods,
-            frequency=frequency,
-            values=values,
-            func=func,
-        )
+        if populate:
+            test = (x is not None for x in (start, periods, values, func))
+            populator = _SERIES_POPULATOR.get(tuple(test), _invalid_constructor)
+            populator(
+                self,
+                start=start,
+                periods=periods,
+                frequency=frequency,
+                values=values,
+                func=func,
+            )
+
+    @classmethod
+    def _guaranteed(
+        klass,
+        start: Period,
+        values: _np.ndarray,
+        description: str = "",
+    ) -> Self:
+        """
+        """
+        self = klass()
+        self.start = start
+        self.data = values
+        self._description = description
+        self.trim()
+        return self
 
     def reset(self, /, ) -> None:
         self.__init__(
@@ -266,7 +297,7 @@ self = Series(
         """
         return _np.full(
             (num_rows, self.shape[1], ),
-            self._missing,
+            _np.nan,
             dtype=self.data_type
         )
 
@@ -299,15 +330,15 @@ self = Series(
     @_pages.reference(category="property", )
     def span(self, ):
         """==Time span of the time series=="""
-        return Span(self.start, self.end_date, ) if self.start else ()
+        return Span(self.start, self.end, ) if self.start else ()
 
     range = span
 
     @property
     @_pages.reference(category="property", )
-    def from_to(self, ):
+    def from_until(self, ):
         """==Two-tuple with the start date and end date of the time series=="""
-        return self.start, self.end_date
+        return self.start, self.end
 
     @property
     @_pages.reference(category="property", )
@@ -324,9 +355,11 @@ self = Series(
         raise NotImplementedError
 
     @property
-    def start_date(self, /, ):
+    def start_period(self, /, ):
         return self.start
 
+    start_date = start_period
+
     @property
     @_pages.reference(category="property", )
     def end(self):
@@ -336,11 +369,12 @@ self = Series(
             if self.start else None
         )
 
+    end_period = end
     end_date = end
 
     @property
     @_pages.reference(category="property", )
-    def frequency(self):
+    def frequency(self, /, ):
         """==Date frequency of the time series=="""
         return (
             self.start.frequency
@@ -348,6 +382,39 @@ self = Series(
             else Frequency.UNKNOWN
         )
 
+    @property
+    @_pages.reference(category="property", )
+    def is_empty(self, ) -> bool:
+        """==True if the time series is empty=="""
+        return not self.data.size
+
+    @property
+    @_pages.reference(category="property", )
+    def has_missing(self, /, ):
+        """==True if the time series is non-empty and contains in-sample missing values=="""
+        return (not self.is_empty) and _np.isnan(self.data).any()
+
+    def any_missing(self, *args, ) -> bool:
+        """
+        """
+        return self._func_missing(_np.any, *args, )
+
+    def all_missing(self, *args, ) -> bool:
+        """
+        """
+        return self._func_missing(_np.all, *args, )
+
+    def count_missing(self, *args, ) -> int:
+        """
+        """
+        return self._func_missing(_np.count_nonzero, *args, )
+
+    def _func_missing(self, func, *args, ) -> bool:
+        """
+        """
+        data = self.get_data(*args, )
+        return func(_np.isnan(data))
+
     def set_data(
         self,
         dates: Dates,
@@ -459,13 +526,31 @@ self = Series(
             raise ValueError("Use expand_num_variants to expand the number of variants")
         self.data = self.data[:, :-remove]
 
-    def get_data(
+    def get_values(
         self,
-        dates = ...,
+        *args,
+        unpack_singleton: bool = True,
+        **kwargs,
+    ) -> _np.ndarray:
+        """
+        """
+        data = self.get_data(*args, **kwargs, )
+        values = [ tuple(data_column.tolist()) for data_column in data.T ]
+        return _has_variants.unpack_singleton(
+            values, self.is_singleton,
+            unpack_singleton=unpack_singleton,
+        )
+
+    def get_data(self, *args, **kwargs, ) -> _np.ndarray:
+        return self.get_data_and_periods(*args, **kwargs, )[0]
+
+    def get_data_and_periods(
+        self,
+        dates: Iterable[Period] | None | EllipsisType = ...,
         *args,
     ) -> _np.ndarray:
-        dates, pos, variants, expanded_values = self._resolve_dates_and_positions(dates, *args, )
-        return expanded_values[_np.ix_(pos, variants)]
+        periods, pos, variants, expanded_values = self._resolve_dates_and_positions(dates, *args, )
+        return expanded_values[_np.ix_(pos, variants)], periods
 
     def get_data_variant(
         self,
@@ -478,28 +563,28 @@ self = Series(
         variant = variant if variant and variant<self.data.shape[1] else 0
         return self.get_data(dates, variant, )
 
-    def get_data_from_to(
+    def get_data_from_until(
         self,
-        from_to,
+        from_until,
         *args,
     ) -> _np.ndarray:
         """
         """
         _, pos, variants, expanded_data \
-            = self._resolve_dates_and_positions(from_to, *args, )
+            = self._resolve_dates_and_positions(from_until, *args, )
         from_pos, to_pos = pos[0], pos[-1]+1
         return expanded_data[from_pos:to_pos, variants]
 
-    def get_data_variant_from_to(
+    def get_data_variant_from_until(
         self,
-        from_to: Iterable[Period],
+        from_until: Iterable[Period],
         variant: int | None = None,
         /,
     ) -> _np.ndarray:
         """
         """
         variant = variant if variant and variant < self.data.shape[1] else 0
-        return self.get_data_from_to(from_to, variant, )
+        return self.get_data_from_until(from_until, variant, )
 
     def extract_variants(
         self,
@@ -520,7 +605,7 @@ self = Series(
         self.start = new_start
         return self
 
-    def _resolve_dates(self, dates, ):
+    def _resolve_dates(self, dates, ) -> tuple[Period]:
         """
         """
         if dates is None:
@@ -602,17 +687,19 @@ self = Series(
         )
         self._replace_data(new_data, )
 
-    def hstack(self, *args):
+    def hstack(self, *args, ):
+        """
+        """
         if not args:
             return self.copy()
         self_args = (self, *args, )
-        if all(i.is_empty() for i in self_args):
+        if all(i.is_empty for i in self_args):
             num_variants = sum(i.num_variants for i in self_args)
             return Series(num_variants=num_variants, )
-        encompassing_span, *from_to = _dates.get_encompassing_span(self, *args, )
-        new_data = self.get_data_from_to(from_to, )
+        encompassing_span, *from_until = _dates.get_encompassing_span(self, *args, )
+        new_data = self.get_data_from_until(from_until, )
         add_data = (
-            x.get_data_from_to(from_to, )
+            x.get_data_from_until(from_until, )
             if hasattr(x, "get_data")
             else _create_data_variant_from_number(x, encompassing_span, self.data_type)
             for x in args
@@ -620,27 +707,53 @@ self = Series(
         new_data = _np.hstack((new_data, *add_data))
         new = Series(num_variants=new_data.shape[1], )
         new.set_data(encompassing_span, new_data, )
-        new.trim()
         return new
 
+    @_pages.reference(category="homogenizing", )
     def clip(
         self,
         /,
         new_start: Period | None,
-        new_end_date: Period | None,
+        new_end: Period | None,
     ) -> None:
+        r"""
+················································································
+
+==Clip time series to a new start and end period==
+
+    self.clip(new_start, new_end)
+
+
+### Input arguments ###
+
+
+???+ input "new_start"
+    The new start period for the `self` time series; if `None`, the current
+    start period is kept.
+
+???+ input "new_end"
+    The new end period for the `self` time series; if `None`, the current
+    end period is kept.
+
+
+### Returns ###
+
+
+???+ returns "None"
+    This method modifies `self` in-place and does not return a value.
+
+
+················································································
+        """
         if new_start is None or new_start < self.start:
             new_start = self.start
-        if new_end_date is None or new_end_date > self.end_date:
-            new_end_date = self.end_date
-        if new_start == self.start and new_end_date == self.end_date:
+        if new_end is None or new_end > self.end:
+            new_end = self.end
+        if new_start == self.start and new_end == self.end:
             return
-        self.data = self.get_data_from_to((new_start, new_end_date, ), )
+        self.data = self.get_data_from_until((new_start, new_end, ), )
         self.start = new_start
 
-    def is_empty(self, ) -> bool:
-        return not self.data.size
-
     def empty(
         self,
         *,
@@ -694,13 +807,59 @@ self = Series(
         self,
         new_date: Period,
         old_date: Period | None = None,
-        /,
     ) -> None:
         """
         """
-        self.start = new_date \
-            if old_date is None \
-            else new_date - (old_date - self.start)
+        if old_data is None:
+            self.start = new_date
+        else:
+            self.start = new_date - (old_date - self.start)
+
+    @_pages.reference(category="manipulation", )
+    def replace_where(
+        self,
+        test: Callable,
+        new_value: Real,
+    ) -> None:
+        """
+················································································
+
+==Replace time series values that pass a test==
+
+```
+self.replace_where(
+    test,
+    new_value,
+)
+```
+
+
+### Input arguments ###
+
+
+???+ input "self"
+    Time series whose observations will be tested and those passing the test
+    replaced.
+
+???+ input "test"
+    A function (or a Callable) that takes a numpy array and returns `True` or
+    `False` for each individual value.
+
+???+ input "new_value"
+    The value to replace the observations that pass the test.
+
+
+### Returns ###
+
+
+???+ returns "None"
+    This method modifies `self` in-place and does not return a value.
+
+
+················································································
+        """
+        self.data[test(self.data)] = new_value
+        self.trim()
 
     def _shallow_copy_data(
         self,
@@ -724,16 +883,16 @@ self = Series(
         if self.data.size == 0:
             self.reset()
             return self
-        num_leading = _get_num_leading_missing_rows(self.data, self._test_missing_period)
-        if num_leading == self.data.shape[0]:
+        num_leading, num_trailing \
+            = _get_num_leading_trailing_missing_rows(self.data, )
+        if num_trailing == self.data.shape[0]:
             self.reset()
             return self
-        num_trailing = _get_num_leading_missing_rows(self.data[::-1], self._test_missing_period)
         if not num_leading and not num_trailing:
             return self
         slice_from = num_leading or None
         slice_to = -num_trailing if num_trailing else None
-        self.data = self.data[slice(slice_from, slice_to), ...]
+        self.data = self.data[slice_from:slice_to, ...]
         if slice_from:
             self.start += int(slice_from)
         return self
@@ -741,7 +900,7 @@ self = Series(
     def _create_expanded_data(self, add_before, add_after):
         return _np.pad(
             self.data, ((add_before, add_after), (0, 0)),
-            mode="constant", constant_values=self._missing
+            mode="constant", constant_values=_np.nan,
         )
 
     def _check_data_shape(self, data, /, ):
@@ -886,12 +1045,12 @@ self = Series(
         if not isinstance(other, type(self)):
             return self.apply(lambda data: func(data, other))
         # FIXME: empty encompassing range
-        _, *from_to = _dates.get_encompassing_span(self, other)
-        self_data = self.get_data_from_to(from_to, )
-        other_data = other.get_data_from_to(from_to, )
+        _, *from_until = _dates.get_encompassing_span(self, other)
+        self_data = self.get_data_from_until(from_until, )
+        other_data = other.get_data_from_until(from_until, )
         new_data = func(self_data, other_data)
         new = Series(num_variants=new_data.shape[1], ) if new is None else new
-        new._replace_start_and_values(from_to[0], new_data, )
+        new._replace_start_and_values(from_until[0], new_data, )
         return new
 
     def _broadcast_variants(self, num_variants, /, ) -> None:
@@ -944,23 +1103,23 @@ self = Series(
     def iter_variants(self, /, ) -> Iterator[Self]:
         """
         """
-        for data in self.iter_data_variants_from_to(..., ):
+        for data in self.iter_data_variants_from_until(..., ):
             new = self.copy()
             new._replace_data(data, )
             yield new
 
-    def iter_own_data_variants_from_to(self, from_to, /, ) -> Iterator[_np.ndarray]:
+    def iter_own_data_variants_from_until(self, from_until, /, ) -> Iterator[_np.ndarray]:
         """
         Iterates over the data variants from the given start date to the given end date
         """
-        data_from_to = self.data if from_to == ... else self.get_data_from_to(from_to, )
-        return iter(data_from_to.T, )
+        data_from_until = self.data if from_until == ... else self.get_data_from_until(from_until, )
+        return iter(data_from_until.T, )
 
-    def iter_data_variants_from_to(self, from_to, /, ) -> Iterator[_np.ndarray]:
+    def iter_data_variants_from_until(self, from_until, /, ) -> Iterator[_np.ndarray]:
         """
         Iterates over the data variants from the given start date to the given end date
         """
-        return _iterators.exhaust_then_last(self.iter_own_data_variants_from_to(from_to, ), )
+        return _iterators.exhaust_then_last(self.iter_own_data_variants_from_until(from_until, ), )
 
     def logistic(self, /, ) -> Self:
         """
@@ -970,18 +1129,32 @@ self = Series(
     for n in FUNCTION_ADAPTATIONS_ELEMENTWISE:
         exec(f"def {n}(self, *args, **kwargs, ): self.data = _ELEMENTWISE_FUNCTIONS['{n}'](self.data, *args, **kwargs, )")
 
+    for n in FUNCTION_ADAPTATIONS_TIMEWISE_NUMPY:
+        exec(f"def {n}(self, *args, **kwargs, ): self.data = _np.{n}(self.data, axis=1, *args, **kwargs, ).reshape(-1, 1)")
+
     #]
 
 
-def _get_num_leading_missing_rows(data, test_missing_period, /, ):
-    try:
-        num = next(
-            i for i, period_data in enumerate(data)
-            if not test_missing_period(period_data)
-        )
-    except StopIteration:
-        num = data.shape[0]
-    return num
+def _get_num_leading_trailing_missing_rows(data: _np.ndarray, /, ):
+    """
+    """
+    #[
+    # Boolean index of rows with at least one observation
+    boolex_observations = ~_np.all(_np.isnan(data, ), axis=1, )
+    if _np.all(boolex_observations, ):
+        # All rows have at least one observation
+        num_leading = 0
+        num_trailing = 0
+    elif not _np.any(boolex_observations, ):
+        # No row has any observation
+        num_leading = 0
+        num_trailing = data.shape[0]
+    else:
+        # Some rows have observations
+        num_leading = _np.argmax(boolex_observations)
+        num_trailing = _np.argmax(boolex_observations[::-1])
+    return num_leading, num_trailing
+    #]
 
 
 def hstack(first, *args) -> Self:
@@ -997,6 +1170,23 @@ def _create_data_variant_from_number(
     return _np.full((len(span), 1), number, dtype=data_type)
 
 
+class series:
+    """
+    """
+    #[
+
+    for n in FUNCTION_ADAPTATIONS_TIMEWISE_NUMPY:
+        exec(_tw.dedent(f"""
+            @staticmethod
+            def {n}(self: Series, *args, **kwargs, ) -> None:
+                new = self.copy()
+                new.{n}(*args, **kwargs, )
+                return new
+        """))
+
+    #]
+
+
 for n in FUNCTION_ADAPTATIONS_ELEMENTWISE:
     exec(
         f"@_ft.singledispatch\n"
